#include <SdFat.h>
#include <Ethernet.h>

SdFat SD;

const int carteSD = 4;
const int led = 2;
const byte LM35 = 0;

byte macSerre[] = {0x90, 0xA2, 0xDA, 0x10, 0x4F, 0x25};
IPAddress IPSerre(192, 168, 1, 200);
EthernetServer serveurHTTP(80);

void setup()
{
  SD.begin(carteSD);
  Ethernet.begin(macSerre, IPSerre);
  serveurHTTP.begin();
  //Serial.begin(115200);
  pinMode(led, OUTPUT);
}

void loop()
{
  EthernetClient client = serveurHTTP.available();
  if (client)
  {
    if (client.connected())
    {
      String reception;
      while (client.available())
      {
        char carLu = client.read();
        if (carLu != 10)
        {
          reception += carLu;
        }
        else
        {
          break;
        }
      }
      if (reception.indexOf("ajax?") != -1) // Si requête ajax
      {
        if (reception.indexOf("maj?") == -1) // Si pas maj
        {
          reception.replace("GET /ajax?", "");
          reception.replace("HTTP/1.1", "");
          reception.trim();
          //Serial.println(reception);
          if (reception == "ON")
          {
            digitalWrite(led, HIGH);
          }
          else if (reception == "OFF")
          {
            digitalWrite(led, LOW);
          }
        }
        arHtml(client, "application/json");
        maj(client);
      }
      else
      {
        //Serial.println(reception);
        if (reception.startsWith("GET / HTTP/1.1"))
        {
          arHtml(client, "text/html");
          envoiFichier(client, "telecommande.html");
        }
        else if (reception.startsWith("GET /telecommande.js HTTP/1.1"))
        {
          arHtml(client, "application/javascript");
          envoiFichier(client, "telecommande.js");
        }
        else if (reception.startsWith("GET /telecommande.css HTTP/1.1"))
        {
          arHtml(client, "text/css");
          envoiFichier(client, "telecommande.css");
        }
        else if (reception.startsWith("GET /telecommande.png HTTP/1.1"))
        {
          arHtml(client, "image/png");
          envoiFichier(client, "telecommande.png");
        }
        delay(1);
      }
      client.stop();
    }
  }
}

// FONCTIONS

void arHtml(EthernetClient nomClient, String type)
{
  nomClient.println(F("HTTP/1.1 200 OK"));
  nomClient.println("Content-Type: " + type);
  nomClient.println(F("Connection: close"));
  nomClient.println();
}

void envoiFichier(EthernetClient nomClient, String fichierEnCours)
{
  char tableau[fichierEnCours.length() + 1];
  fichierEnCours.toCharArray(tableau, fichierEnCours.length() + 1);
  if (SD.exists(tableau))
  {
    File fichier = SD.open(fichierEnCours, FILE_READ);
    while (fichier.available())
    {
      nomClient.write(fichier.read());
    }
    fichier.close();
  }
//  else
//  {
//    nomClient.println("Page '" + fichierEnCours + "' introuvable...");
//  }
}

void maj(EthernetClient nomClient)
{
  String reponseJSON = "{";
  reponseJSON.concat("\"bouton\":");
  if (digitalRead(led) == HIGH) reponseJSON.concat("\"ledOn\","); 
  else reponseJSON.concat("\"ledOff\",");
  reponseJSON.concat("\"temperature\":");
  reponseJSON.concat(temperature(LM35, 100));
  reponseJSON.concat("}");
  nomClient.println(reponseJSON);
}

float temperature(byte broche, word iteration)
{
  long lecture = 0;
  for (word i = 0; i < iteration; i++)
  {
    lecture += analogRead(broche);
  }   
  return(float(lecture) *500 /1024 /iteration);
}

